@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2012 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -48,7 +48,7 @@ typedef signed char depth_t;
 //! Class for getting access to drawing memory
 class drawing_memory
 {
-#ifdef __TBB_MIC
+#ifdef __TBB_MIC_OFFLOAD
     // The address is kept as uintptr_t since
     // the compiler could not offload a pointer
 #endif
@@ -171,12 +171,17 @@ inline color_t video::get_color(colorcomp_t red, colorcomp_t green, colorcomp_t
     if(red_shift == 16) // only for depth == 24 && red_shift > blue_shift
         return (red<<16) | (green<<8) | blue;
     else if(depth >= 24)
-        return (red<<red_shift) | (green<<green_shift) | (blue<<blue_shift);
+        return
+#if __ANDROID__
+                // Setting Alpha to 0xFF
+                0xFF000000 |
+#endif
+                (red<<red_shift) | (green<<green_shift) | (blue<<blue_shift);
     else if(depth > 0) {
         register depth_t bs = blue_shift, rs = red_shift;
         if(blue_shift < 0) blue >>= -bs, bs = 0;
         else /*red_shift < 0*/ red >>= -rs, rs = 0;
-        return (red<<rs)&red_mask | (green<<green_shift)&green_mask | (blue<<bs)&blue_mask;
+        return ((red<<rs)&red_mask) | ((green<<green_shift)&green_mask) | ((blue<<bs)&blue_mask);
     } else { // UYVY colorspace
         register unsigned y, u, v;
         y = red * 77 + green * 150 + blue * 29; // sum(77+150+29=256) * max(=255):  limit->2^16
