@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2012 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks.
 
@@ -56,9 +56,7 @@
 */
 
 /* 
- * parse.c - an UltraLame (tm) parser for simple data files...
- *
- *  $Id: parse.cpp,v 1.3 2007-02-22 17:54:15 Exp $
+ * parse.cpp - an UltraLame (tm) parser for simple data files...
  */
 
 // Try preventing lots of GCC warnings about ignored results of fscanf etc.
@@ -95,7 +93,7 @@ static int numobjectsparsed;    /* total number of objects parsed so far   */
 static color scenebackcol;   /* scene background color                  */
 
 static int stringcmp(const char * a, const char * b) {
-  int i, s, l;
+  size_t i, s, l;
 
   s=strlen(a);
   l=strlen(b);
