@@ -56,9 +56,7 @@
 */
 
 /* 
- * tgafile.c - This file contains the code to write 24 bit targa files...
- *
- *  $Id: tgafile.cpp,v 1.2 2007-02-22 17:54:16 Exp $
+ * tgafile.cpp - This file contains the code to write 24 bit targa files...
  */
 
 #include "machine.h"
@@ -133,7 +131,8 @@ void writetgaregion(void * voidofp,
                     int stopx, int stopy, char * buffer) {
   int y, totalx, totaly;
   char * bufpos;
-  int filepos, numbytes;
+  long filepos;
+  size_t numbytes;
   FILE * ofp = (FILE *) voidofp;
  
   totalx = stopx - startx + 1;
@@ -141,7 +140,7 @@ void writetgaregion(void * voidofp,
 
   for (y=0; y<totaly; y++) {
     bufpos=buffer + (totalx*3)*(totaly-y-1);
-    filepos=18 + iwidth*3*(iheight - starty - totaly + y + 1) + (startx - 1)*3; 
+    filepos=18 + iwidth*3*(iheight - starty - totaly + y + 1) + (startx - 1)*3;
 
     if (filepos >= 18) {
       fseek(ofp, filepos, 0); 
@@ -149,7 +148,7 @@ void writetgaregion(void * voidofp,
 
       if (numbytes != totalx) {
         char msgtxt[256];
-	sprintf(msgtxt, "File write problem, %d bytes written.", numbytes);  
+        sprintf(msgtxt, "File write problem, %d bytes written.", (int)numbytes);
         rt_ui_message(MSG_ERR, msgtxt);
       }
     }
@@ -163,7 +162,8 @@ void writetgaregion(void * voidofp,
 
 int readtga(char * name, int * xres, int * yres, unsigned char **imgdata) {
   int format, width, height, w1, w2, h1, h2, depth, flags;
-  int imgsize, bytesread, i, tmp;
+  int imgsize, i, tmp;
+  size_t bytesread;
   FILE * ifp;
 
   ifp=fopen(name, "r");  
@@ -226,7 +226,7 @@ int readtga(char * name, int * xres, int * yres, unsigned char **imgdata) {
   for (i=0; i<imgsize; i+=3) {
     tmp = (*imgdata)[i]; /* Blue */
     (*imgdata)[i] = (*imgdata)[i+2]; /* Red */
-    (*imgdata)[i+2] = tmp; /* Blue */    
+    (*imgdata)[i+2] = tmp; /* Blue */
   }
 
   *xres = width;
