@@ -43,7 +43,7 @@ public:
     typedef blocked_range<PageValue> page_range_type;
     typedef blocked_range<RowValue>  row_range_type;
     typedef blocked_range<ColValue>  col_range_type;
- 
+
 private:
     page_range_type my_pages;
     row_range_type  my_rows;
@@ -53,16 +53,16 @@ public:
 
     blocked_range3d( PageValue page_begin, PageValue page_end,
                      RowValue  row_begin,  RowValue row_end,
-                     ColValue  col_begin,  ColValue col_end ) : 
+                     ColValue  col_begin,  ColValue col_end ) :
         my_pages(page_begin,page_end),
         my_rows(row_begin,row_end),
         my_cols(col_begin,col_end)
     {
     }
 
-    blocked_range3d( PageValue page_begin, PageValue page_end, typename page_range_type::size_type page_grainsize, 
+    blocked_range3d( PageValue page_begin, PageValue page_end, typename page_range_type::size_type page_grainsize,
                      RowValue  row_begin,  RowValue row_end,   typename row_range_type::size_type row_grainsize,
-                     ColValue  col_begin,  ColValue col_end,   typename col_range_type::size_type col_grainsize ) :  
+                     ColValue  col_begin,  ColValue col_end,   typename col_range_type::size_type col_grainsize ) :
         my_pages(page_begin,page_end,page_grainsize),
         my_rows(row_begin,row_end,row_grainsize),
         my_cols(col_begin,col_end,col_grainsize)
@@ -80,37 +80,57 @@ public:
         return  my_pages.is_divisible() || my_rows.is_divisible() || my_cols.is_divisible();
     }
 
-    blocked_range3d( blocked_range3d& r, split ) : 
+    blocked_range3d( blocked_range3d& r, split ) :
         my_pages(r.my_pages),
         my_rows(r.my_rows),
         my_cols(r.my_cols)
     {
-        if( my_pages.size()*double(my_rows.grainsize()) < my_rows.size()*double(my_pages.grainsize()) ) {
+        split split_obj;
+        do_split(r, split_obj);
+    }
+
+#if !TBB_DEPRECATED
+    //! Static field to support proportional split
+    static const bool is_divisible_in_proportion = true;
+
+    blocked_range3d( blocked_range3d& r, proportional_split& proportion ) :
+        my_pages(r.my_pages),
+        my_rows(r.my_rows),
+        my_cols(r.my_cols)
+    {
+        do_split(r, proportion);
+    }
+#endif
+
+    template <typename Split>
+    void do_split( blocked_range3d& r, Split& split_obj)
+    {
+        if ( my_pages.size()*double(my_rows.grainsize()) < my_rows.size()*double(my_pages.grainsize()) ) {
             if ( my_rows.size()*double(my_cols.grainsize()) < my_cols.size()*double(my_rows.grainsize()) ) {
-                my_cols.my_begin = col_range_type::do_split(r.my_cols);
+                my_cols.my_begin = col_range_type::do_split(r.my_cols, split_obj);
             } else {
-                my_rows.my_begin = row_range_type::do_split(r.my_rows);
+                my_rows.my_begin = row_range_type::do_split(r.my_rows, split_obj);
             }
 	} else {
             if ( my_pages.size()*double(my_cols.grainsize()) < my_cols.size()*double(my_pages.grainsize()) ) {
-                my_cols.my_begin = col_range_type::do_split(r.my_cols);
+                my_cols.my_begin = col_range_type::do_split(r.my_cols, split_obj);
             } else {
-                    my_pages.my_begin = page_range_type::do_split(r.my_pages);
+                my_pages.my_begin = page_range_type::do_split(r.my_pages, split_obj);
             }
         }
     }
 
-    //! The pages of the iteration space 
+    //! The pages of the iteration space
     const page_range_type& pages() const {return my_pages;}
 
-    //! The rows of the iteration space 
+    //! The rows of the iteration space
     const row_range_type& rows() const {return my_rows;}
 
-    //! The columns of the iteration space 
+    //! The columns of the iteration space
     const col_range_type& cols() const {return my_cols;}
 
 };
 
-} // namespace tbb 
+} // namespace tbb
 
 #endif /* __TBB_blocked_range3d_H */
